from ants.utils import BaseMixin


class Rule(object):

    def __call__(self):
        pass


class Parser(BaseMixin):

    def rule_1(self, obj):
        print("rule_1")
        return 

    @property
    def rules(self):
        return []


import os
import sys
import ants


def print_help():
    print("help is not complete yeah")


def start_project(argv):
    if len(sys.argv) <= 2:
        print("[Error:A project_name is requried]")
        return
    project_name = sys.argv[2]
    template_path = "%s/conf/project_template/" % ants.__path__[0]
    os.system("django-admin startproject --template %s %s" %
              (template_path, project_name))


if __name__ == '__main__':
    if len(sys.argv) <= 1:
        print_help()
    command = sys.argv[1]
    if command == "startproject":
        start_project(sys.argv)
    else:
        print_help()



from gevent import monkey
from django.db import transaction
import gevent
import time


monkey.patch_all(socket=True, dns=True, time=True, select=True,
                 thread=False, os=True, ssl=True, httplib=False, aggressive=True)


def group_list(_list, is_size=True):
    def with_size(size):
        count = int(len(_list) / size) + 1
        return [_list[i * size:(i + 1) * size] for i in range(count)]
    def with_count(count):
        return [_list[i::count] for i in range(count)]
    return with_size if is_size else with_count


class BaseMixin(object):
    task_model = ''
    task_model_key = 'id'
    goal_model = ''
    goal_model_key = 'id'
    related_key = ''
    container_size = 1000
    thread = 1

    def __init__(self):
        if not self.related_key:
            self.related_key = self.task_model_key

    @property
    def done_task(self):
        if not self.goal_model:
            return {}
        ids = self.goal_model.objects.all().values_list(self.related_key)
        return {"%s__in" % self.related_key: ids}

    @property
    def all_task_obj(self):
        return self.task_model.objects.filter(**self.task_obj_filter(()))

    @property
    def task_ids(self):
        objs = self.all_task_obj.exclude(**self.done_task)
        ids = objs.values_list(self.task_model_key)
        return list(map(lambda x: x[0], ids))

    def task_obj_filter(self):
        return {}

    def task_filter(self, task):
        return True

    def run(self, task):
        pass

    def multi_update(self, bulk_id):
        objs = self.task_model.objects.filter(
            **{"%s__in" % self.task_model_key: bulk_id})
        objs = filter(self.task_filter, objs)
        list(map(self.run, objs))

    @transaction.atomic
    def multi_run(self, bulk_ids):
        for bulk_id in bulk_ids:
            self.multi_update(bulk_id)

    def run_all(self):
        start_time = time.time()
        bulk_ids = group_list(self.task_ids)(self.container_size)
        bulk_id_group = group_list(bulk_ids, False)(self.thread)
        jobs = [gevent.spawn(self.multi_run, id_group)
                for id_group in bulk_id_group]
        gevent.joinall(jobs)
        end_time = time.time()
        print("using %s(s)" % (end_time - start_time))

    def start(self):
        self.run_all()



import codecs
import os
import sys
from setuptools import setup, find_packages


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


def readme():
    return read("./README.md")

VERSION = "0.0.4"

CONF_PATH = "ants/conf"

requires = ["ants==0.0.1", "Django==1.10.4", "gevent==1.1.2", "Jinja2==2.8"]

datafiles = [(root, [os.path.join(root, f) for f in files])
             for root, dirs, files in os.walk(CONF_PATH)]

datafiles.append(('./',['README.md']))

print(datafiles)

setup(
    name='ants',
    version=VERSION,
    description='A human spider framework base on django',
    long_description=readme().strip(),
    author='mymusise',
    author_email='mymusise1@gmail.com',
    url='https://github.com/mymusise/ants',
    license='MIT',
    scripts=['bin/ants-tools.py'],
    packages=find_packages(),
    install_requires=[],
    keywords='spider django human',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development'
    ],
    zip_safe=False,
    data_files=datafiles,
    # install_requires=requires,
)



from django.core.management.base import BaseCommand
from importlib import import_module
import os


class BaseAntCommand(BaseCommand):
    app_name = ''


    def add_arguments(self, parser):
        parser.add_argument('args', metavar='app_label', nargs='*',
                            help='Specify the app label(s) to create migrations for.')

    def not_cache_file(self, file):
        return False if "__pycache" in file or ".pyc" in file else True

    def get_file_name(self, file):
        try:
            file_name, extension = file.split('.')
        except ValueError:
            raise ValueError(
               "Error loading object '{}': not a full path".format(file))
            return False
        return file_name, extension

    def is_py_file(self, extension):
        return True if extension == 'py' else False

    def get_file_obj(self, file_name):
        return import_module('{}.ants.{}'.format(self.app_name, file_name))

    def handle(self, *args, **options):
        files = os.listdir('{}/ants/'.format(self.app_name))
        files = list(filter(self.not_cache_file, files))
        # map( ,args)
        for name in args:
            for file in files:
                file_name, extension = self.get_file_name(file)
                if not self.is_py_file(extension):
                    continue
                file_obj = self.get_file_obj(file_name)
                _vars = dir(file_obj)
                for _var in _vars:
                    attr = getattr(file_obj, _var)
                    if not hasattr(attr, 'NAME'):
                        continue
                    value = getattr(attr, 'NAME')
                    if value != name:
                        continue
                    obj = attr()
                    print("runing the clawer in [{}]".format(file_obj.__name__))
                    try:
                        return obj.start()
                    except:
                        import traceback
                        traceback.print_exc()
            print("[Ant not found]")
